@@ -7,11 +7,15 @@ import copy
 import os
 import re
 import shutil
+import statistics
 import sys
 import time
 import timeit
 from logging import getLogger, Formatter, StreamHandler
 from typing import Any, List, Optional, Tuple
+from types import FunctionType
+from urllib.request import urlopen
+from zipfile import ZipFile
 
 import numpy as np
 import pandas as pd
@@ -1946,3 +1950,132 @@ def toc(message=None, time_format=None):
         print(f"{message}{time_taken:{time_format}} seconds")
 
     return time_taken 
+
+def download(url, outdir=".", outfile=None, binary=True, unzip=False):
+    """
+    Download data from specified URL.
+
+    **Parameters:**
+
+    url : str
+        URL from which to download data.
+
+    outdir : str/pathlib.Path, default="."
+        Path to (local) directory to which data are to be downloaded.
+
+    outfile : str, default=None
+        Name of the file to be downloaded.  If None, the name is taken
+        to be the part of the URL following the last (non-trailing) slash.
+
+    binary : bool, default=True
+        If True, treat downloaded file as being in binary format.
+
+    unzip : bool, default=True
+        If True, treat downloaded file as being a zip-format archive,
+        and unzip.
+    """
+    # Ensure that output directory exists.
+    outdir = Path(fullpath(outdir))
+    outdir.mkdir(parents=True, exist_ok=True)
+
+    # Retreive data from URL.
+    response = urlopen(url)
+    data = response.read()
+
+    # Write data.
+    outfile = outfile or Path(url).name
+    outpath = outdir / outfile
+    write_opts = "wb" if binary else "w"
+    with open(outpath, write_opts) as outdata:
+        outdata.write(data)
+
+    # Unzip archive.
+    if unzip:
+        with ZipFile(outpath) as zipfile:
+            zipfile.extractall(outdir)
+
+def get_stat(values=None, value_for_none=None, stat="mean", **kwargs):
+    """
+    Calculate statistic(s) for values in a list or dictionary.
+
+    If input values are lists or tuples, a list is returned with
+    element-by-element statistics.
+
+    **Parameters:**
+
+    values: list/tuple/dict
+        Values for which to calculate statistic(s).  If a dictionary,
+        the dictionary values are used.
+
+    value_for_none: int/float/None, default=None
+        Value to be substituted for any None values, before calculation
+        of statistic(s).  If None, None values are omitted, rather than
+        being substituted.
+
+    stat: str, default="mean"
+        Statistic(s) to be calculated.  This should be the name of the
+        function for calculation of the statistic(s) in the Python
+        statistics module:
+
+            https://docs.python.org/3/library/statistics.html
+
+        Available options include: "mean", "median", "mode", "stdev",
+        "quantiles".
+
+    kwargs : dict, default=None
+        Keyword arguments to be passed to the relevant function of
+        the Python statistics module:
+
+            https://docs.python.org/3/library/statistics.html
+
+        For example, if quantiles are required for 10 intervals,
+        rather than for the default of 4, this can be specified using:
+
+        kwargs={"n" : 10}
+    """
+    logger = get_logger(identifier="funcName")
+
+    if isinstance(values, dict):
+        values = list(values.values())
+
+    if not values:
+        logger.warning("No input values: returning None")
+        return
+
+    if not is_list(values[0]):
+        if value_for_none is None:
+            values = [value for value in values if value is not None]
+        else:
+            values = [(value if value is not None else value_for_none)
+                    for value in values]
+        try:
+            return getattr(statistics, stat)(values, **kwargs)
+        except statistics.StatisticsError as error:
+            logger.warning(f"{error}: returning None")
+            return
+
+    if value_for_none is None:
+        components = [[value[idx] for value in values if value[idx] is not None]
+                for idx in range(len(values[0]))]
+    else:
+        components = [[(value[idx] if value[idx] is not None
+            else value_for_none) for value in values]
+            for idx in range(len(values[0]))]
+    try:
+        return [(get_stat(component_values, value_for_none, stat,
+            **(kwargs or {})) if component_values else value_for_none)
+            for component_values in components]
+    except statistics.StatisticsError as error:
+        logger.warning(f"{error}: returning None")
+
+def get_stat_functions():
+    """
+    Get names of statistical functions implemented in Python statistics module.
+
+    For details of statistics module, see:
+
+    https://docs.python.org/3/library/statistics.html
+    """
+    return [function for function in dir(statistics)
+            if type(getattr(statistics, function)) == FunctionType
+            and not function.startswith("_")]
