@@ -2,6 +2,7 @@
 import matplotlib.pyplot as plt
 import matplotlib.cm
 import matplotlib.colors
+import numbers
 import numpy as np
 import os
 from pathlib import Path
@@ -254,7 +255,7 @@ class Registration(Data):
             im = skrt.image.Image(im)
         path = getattr(self, f"{category}_path")
         if not os.path.exists(path) or force:
-            skrt.image.Image.write(im, path, verbose=(self.logger.level < 30))
+            skrt.image.Image.write(im, path, verbose=(self.logger.level < 20))
         if 'grid' in category or 'mask' in category:
             setattr(self, f"{category}", skrt.image.Image(path))
         else:
@@ -745,18 +746,25 @@ class Registration(Data):
 
         # Run
         cmd = self.get_ELASTIX_cmd(step, use_previous_tfile)
-        self.logger.info(f"Running command:\n {' '.join(cmd)}")
+        self.logger.debug(f"Running command:\n {' '.join(cmd)}")
         code = subprocess.run(
                 cmd, capture_output=self.capture_output).returncode
 
         # Check whether registration succeeded
         if code:
             logfile = os.path.join(self.outdirs[step], "elastix.log")
-            self.logger.warning(
+            """
+            self.logger.error(
                 f"Registration step {step} failed! See "
                 f"{logfile} or run Registration.print_log({step}) for "
                 " more info."
             )
+            """
+            raise RuntimeError(
+                f"Registration step {step} failed! See "
+                f"{logfile} or run Registration.print_log({step}) for "
+                " more info."
+                    )
         else:
             self.tfiles[step] = os.path.join(
                 self.outdirs[step], "TransformParameters.0.txt"
@@ -859,7 +867,7 @@ class Registration(Data):
             im_path = im.path
         else:
             im_path = os.path.join(self._tmp_dir, "image.nii.gz")
-            im.write(im_path, verbose=(self.logger.level < 30))
+            im.write(im_path, verbose=(self.logger.level < 20))
 
         # Transform the nifti file
         result_path = self.transform_data(im_path, step, params)
@@ -927,7 +935,7 @@ class Registration(Data):
             "-tp",
             tfile,
         ]
-        self.logger.info(f'Running command:\n {" ".join(cmd)}')
+        self.logger.debug(f'Running command:\n {" ".join(cmd)}')
         code = subprocess.run(
                 cmd, capture_output=self.capture_output).returncode
 
@@ -947,7 +955,7 @@ class Registration(Data):
         return os.path.join(self._tmp_dir, outfile)
 
     def transform_roi(self, roi, step=-1, outfile=None, params=None,
-            transform_points=False):
+            transform_points=False, require_contours=False):
         """Transform a single ROI using the output transform from a given
         registration step (by default, the final step). If the registration
         step has not yet been performed, the step and all preceding steps
@@ -982,6 +990,14 @@ class Registration(Data):
            frame of the fixed image.  If True, the transform is applied
            to push ROI contour points from the reference frame of the
            fixed image to the reference frame of the moving image.
+
+        require_contours : bool, default=False
+           If the transformed ROI doesn't have contours defined: return
+           the ROI if <require_contours> is False; return None if
+           <require_contours> is True.  A transformed ROI won't have
+           contours defined if, for example, it's transformed as a mask,
+           and the non-zero regions of the transformed mask are outside
+           the fixed image.
         """
 
         # Save ROI temporarily as nifti if needed
@@ -994,19 +1010,16 @@ class Registration(Data):
         else:
             ext = 'txt' if transform_points else 'nii.gz'
             roi_path = os.path.join(self._tmp_dir, f"{roi.name}.{ext}")
-            roi.write(roi_path, verbose=(self.logger.level < 30))
+            roi.write(roi_path, verbose=(self.logger.level < 20))
 
         # Set default parameters
         default_params = {"ResampleInterpolator": '"FinalNearestNeighborInterpolator"'}
         if params is not None:
             default_params.update(params)
 
-        # Transform the nifti file or point cloud
-        result_path = self.transform_data(roi_path, step, default_params)
-        if result_path is None or not os.path.exists(str(result_path)):
-            return
-
         # Identify image to be associated with the transformed ROI.
+        # This needs to be here, to avoid the possibility of
+        # the ROI transform result being overwritten by an image transform.
         if transform_points:
             if issubclass(skrt.image.Image, type(self.moving_source)):
                 image = self.moving_source
@@ -1017,9 +1030,14 @@ class Registration(Data):
         else:
             image = self.get_transformed_image(step)
 
+        # Transform the nifti file or point cloud
+        result_path = self.transform_data(roi_path, step, default_params)
+        if result_path is None or not os.path.exists(str(result_path)):
+            return
+
         # Create ROI object, and check that it has contours defined.
         roi = ROI(result_path, name=roi.name, color=roi.color, image=image)
-        if not roi.get_contours():
+        if require_contours and not roi.get_contours():
             return
 
         # Copy to output dir if outfile is set
@@ -1081,7 +1099,7 @@ class Registration(Data):
 
         # Write structure set if outname is given
         if outfile is not None:
-            final.write(outfile, verbose=(self.logger.level < 30))
+            final.write(outfile, verbose=(self.logger.level < 20))
             return
 
         # Otherwise, return structure set
@@ -1122,7 +1140,7 @@ class Registration(Data):
         # If forcing and registration had already been done, re-transform the 
         # moving image (otherwise, moving image will have just been recreated
         # anyway by running registration)
-        if force and was_registered:
+        if (force or step not in self.transformed_images) and was_registered:
             self.transform_moving_image(step)
 
         # Return clone of the transformed image object
@@ -1530,7 +1548,7 @@ class Registration(Data):
             "-tp",
             tfile
         ]
-        self.logger.info(f'Running command:\n {" ".join(cmd)}')
+        self.logger.debug(f'Running command:\n {" ".join(cmd)}')
         code = subprocess.run(cmd, capture_output=self.capture_output).returncode
         if code:
             logfile = os.path.join(outdir, 'transformix.log')
@@ -1542,6 +1560,28 @@ class Registration(Data):
         assert os.path.exists(output_file)
         return dtype(output_file, image=image, title=title)
 
+    def get_mutual_information(self, step=-1, force=False, **kwargs):
+        """
+        For fixed image and transformed moving image after step,
+        calculate mutual information or a variant.
+        after step.
+
+        **Parameters:**
+        step : int/str/list, default=None
+            Name or number of the step for which the registration should be
+            performed. Available steps are listed in self.steps.
+
+        force : bool, default=False
+            If True, transformation of the moving image will be
+            performed, even if the image was transformed previously.
+
+        **kwargs
+            Keyword arguments passed to
+            skrt.image.Image.get_mutual_information()
+            See this method's documentation for options.
+        """
+        return self.fixed_image.get_mutual_information(
+                self.get_transformed_image(step, force), **kwargs)
 
 class Grid(ImageOverlay):
 
